/***************************************************************************
 *   Copyright (C) 2005-2006 by Steffen Wendzel                            *
 *   cdp {at} doomed-reality {dot} org                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <stdio.h>
#include <string.h>
#ifdef __svr4__
	#include <strings.h>
#endif
#include <unistd.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <syslog.h>
#include <stdarg.h>
#include <time.h>
#include <signal.h>

#define FUPIDS_VERSION	"0.8.5"

#define CONFIGFILE	"/etc/fupids2.conf"
#define USERDBFILE	"/var/fupids2/fupids2.userdb"
#define LOGFILE		"/var/fupids2/fupids2.log"
#define INCOMING_DIR	"/var/fupids2/reports"

#define SYSL_NOTICE	LOG_DAEMON|LOG_NOTICE
#define SYSL_ERR	LOG_DAEMON|LOG_ERR

/* ------------------------------------------------------------------ structs */
struct ws_info {
	char		*fqdn;
	char		*house;		/* building */
	unsigned int	 etage;
	unsigned int	 room;
	char             x,y;
	struct ws_info	*next;
};
typedef struct ws_info ws_info;

struct user {
/* typical change factors */
	float avg_house_chng_factor;
	float avg_etage_chng_factor;
	float avg_room_chng_factor;
	float avg_x_chng_factor;
	float avg_y_chng_factor;
	float avg_prog_chng_factor;
/* state values */
	unsigned short last_x;
	unsigned short last_y;
	char *last_house;
	unsigned int last_room;
	unsigned int last_etage;
	unsigned int known_timeintervals[24];
   unsigned int known_daysofweek[7]; // 0 == 'Sunday', 6 == 'Saturday'
	unsigned int num_of_progs;
	unsigned int log_entries;	// # of log entries we already received
/* other stuff */
	char username[512+1];
	bool already_changed;		// did we already incremented 'log_entries' for this report-file?
	float cur_atklvl;		// current attacker level
	struct proglist *users_progs;
	struct user *next;
};
typedef struct user user;

struct proglist {
	char *name;
	float usage_factor;
	unsigned int used;
	char already_logged;
	struct proglist *next;
};
typedef struct proglist proglist;


/* --------------- file formats */
typedef struct {
	unsigned short last_x;
	unsigned short last_y;
	unsigned int last_room;
	unsigned int last_etage;
	unsigned int num_of_progs;
	unsigned long int log_entries;
	unsigned int last_house_len;
	unsigned int known_timeintervals[24];
   unsigned int known_daysofweek[7]; // 0 == 'Sunday', 6 == 'Saturday'
	char username[512+1];
	//char *last_house;
} usrdb_ffmt;

typedef struct {
	unsigned int namelen;
	unsigned int used;
	char already_logged;
} usrdb_prg_ffmt;

/* -------------------------------------------------------- class definitions */
class Workstations {
public:
	ws_info *cur_ws;		// if search_ws() found a WS, cur_ws points to this WS
	
	Workstations();
	int search_ws(char *fqdn);
	void dump_wslist();
private:
	ws_info *ws_db;
	
	void load_base_config();
};

class Users {
public:
	user *user_db;
	user *cur_user;
	struct proglist *cur_prog;

	Users();
	void dump_userlist();
	void lookup_user(char *username);
	void lookup_tool(char *progname);
	void calc_atklvl(ws_info *cur_ws);
	void load_user_config();
	void save_user_config();
};

class Directory {
public:
	struct dirent *dp;
	char username[512+1];
	char cmdname[8192+1];
	
	bool next_element();
	Directory();
private:
	DIR *dirfp;
};

class Log {
public:
	Log();
	~Log();
	void writeline(char *username, float atklvl);
	void writeline(char *username, char *progname, char flag);
private:
	FILE *fp;
};
/* --------------------------------------------- global function declarations */
int get_openfilelen(FILE *fp);
void fflush_out();

