/***************************************************************************
 *   Copyright (C) 2005 by Steffen Wendzel                                 *
 *   Copyright (C) 2005-2006 by Steffen Wendzel                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "fupids2.h"

Users USRS;
Log LOG;

void
sig_handler(int signr)
{
	syslog(SYSL_NOTICE, "clean exit after signal");
	USRS.save_user_config();
#ifdef DEBUG
	printf("[signal handler]\n");
#endif
	exit(0);
}

/* What i learned today in japanish lesson:
 *
 *> Sumimasen. Rajio o misete kudasai.
 *< Dono rajio desu ka.
 *> Ano chiisai rajio desu.
 *< Hai, Dzo.
 *				11/29/2005
 */

int
main(int argc, char *argv[])
{
	FILE *hostfile;
	Directory REPDIR; /* reports-directory */
	Workstations WS;
	pid_t pid;
	
	/* files are located in /tmp/
	 * with names like /tmp/fupids2.report.`hostname`
	 */
	if(chdir(INCOMING_DIR)==-1) {
		perror("cd: " INCOMING_DIR);
		fprintf(stderr, "Please create or change permissions for " INCOMING_DIR "\n");
		fflush(stderr);
		exit(1);
	}
	
	if(signal(SIGINT, sig_handler)==SIG_ERR) {
		perror("signal");
		exit(1);
	}
	if(signal(SIGTERM, sig_handler)==SIG_ERR) {
		perror("signal");
		exit(1);
	}
	
	if(argc>=2 && argv[1][0]=='f') { /* foreground mode */
		printf("fupids2 v" FUPIDS_VERSION " ready. (foreground-mode)\n");
	} else { /* daemon mode */
		if((pid=fork())<0) {
			perror("fork");
			return 1;
		} else if(pid) { /* parent */
			return 0;
		}
		setsid();
		umask(0);
		syslog(SYSL_NOTICE, "fupids2 v" FUPIDS_VERSION " ready. (daemon-mode)\n");
	}
	
	while(1) {
		while(REPDIR.next_element()!=false) {
			if((hostfile=fopen(REPDIR.dp->d_name, "r"))==NULL) {
				syslog(SYSL_ERR, "Unable to open %s\n", REPDIR.dp->d_name);
				fprintf(stderr, "fupids2: unable to open %s.\n", REPDIR.dp->d_name);
			} else {
				if(WS.search_ws(REPDIR.dp->d_name)==-1) {
					syslog(SYSL_NOTICE, "Host %s is not in the Database.", REPDIR.dp->d_name);
				} else {
					while(fscanf(hostfile, "%512s %8192s\n", REPDIR.username, REPDIR.cmdname)!=EOF) {
						USRS.lookup_user(REPDIR.username);
						USRS.lookup_tool(REPDIR.cmdname);
						
					}
					for(USRS.cur_user=USRS.user_db; USRS.cur_user; USRS.cur_user=USRS.cur_user->next)
						if(USRS.cur_user->already_changed==true) {
							USRS.calc_atklvl(WS.cur_ws);
							if(USRS.cur_user->cur_atklvl>=1) {
								LOG.writeline(USRS.cur_user->username, USRS.cur_user->cur_atklvl);
#ifdef DEBUG
								printf("[wrote log]\n");
#endif
								fflush(stdout);
							}
						}
				}
				fclose(hostfile);
				unlink(REPDIR.dp->d_name);
			}
		}
		sleep(10);
	}
	return 0;
}

