/***************************************************************************
 *   Copyright (C) 2005-2006 by Steffen Wendzel                            *
 *   cdp {at} doomed-reality {dot} org                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "fupids2.h"

extern Log LOG;
extern int atk_roomchg,
    atk_seatchg,
    atk_etagechg,
    atk_buildingchg,
    atk_tools,
    atk_time,
    atk_date;
struct tm *tm_;

Users::Users()
{
	user_db=cur_user=NULL;
	load_user_config();
#ifdef DEBUG
	dump_userlist();
#endif
}

void
Users::dump_userlist()
{
	user *csr;
	proglist *prg;
	
	for(csr=user_db; csr; csr=csr->next) {
		printf("USERNAME=%s\n\ttools=", csr->username);
		for(prg=csr->users_progs; prg; prg=prg->next)
			printf("%s,", prg->name);
		putchar('\n');
	}
}


// 
// calculate the users current attacker level
// 

#define SET_SOCALLED_LAST_ENTRIES		\
	cur_user->last_y=cur_ws->y;		\
	cur_user->last_x=cur_ws->x;		\
	cur_user->last_house=cur_ws->house;	\
	cur_user->last_room=cur_ws->room;	\
	cur_user->last_etage=cur_ws->etage;

void
Users::calc_atklvl(ws_info *cur_ws)
{
	float A3,A4,A5;
	// neurons
	float h1,h2,h3,h4, po1, po2, out;
	
	// get the current time
	//tm_=localtime(&x);  // lookup_user will do this for me
	
	atk_roomchg=atk_seatchg=atk_etagechg=atk_buildingchg=atk_tools=atk_time=atk_date=0;
	
// 	float avg_prog_chng_factor;
	A3=A4=A5=1.0f;
	h1=h2=h3=h4=po1=po2=out=0.f;

	/* simply compare pointer addresses */
	if(cur_user->last_house==cur_ws->house) {
		cur_user->avg_house_chng_factor=0;
	} else {
		cur_user->avg_house_chng_factor=2;
		atk_buildingchg=1;
	}
	
	if(cur_ws->etage==cur_user->last_etage) {
		cur_user->avg_etage_chng_factor=0;
	} else {
		cur_user->avg_etage_chng_factor=1;
		atk_etagechg=1;
	}
	
	if(cur_ws->room==cur_user->last_room) {
		cur_user->avg_room_chng_factor=0;
	} else {
		cur_user->avg_room_chng_factor=1;
		atk_roomchg=1;
	}

	if(cur_user->last_x==cur_ws->x) {
		cur_user->avg_x_chng_factor=0;
	} else {
		cur_user->avg_x_chng_factor=1;
		atk_seatchg=1;
	}

	if(cur_user->last_y==cur_ws->y) {
		cur_user->avg_y_chng_factor=0;
	} else {
		cur_user->avg_y_chng_factor=1;
		atk_seatchg=1;
	}
	
	// now set the current infos as 'last' informations
	SET_SOCALLED_LAST_ENTRIES

	// ///////////////////////////////
	// CALC INPUT NN
	// ///////////////////////////////
	// tool-profile
	for(A3=0.f,cur_prog=cur_user->users_progs; cur_prog; cur_prog=cur_prog->next) {
		cur_prog->usage_factor=((float)cur_user->log_entries/cur_prog->used);
		if(cur_prog->usage_factor>=3.33f) {
#ifdef DEBUG
			printf("\n\t\t\t%s==%f, calc=%i/%i\n", cur_prog->name, cur_prog->usage_factor,
						cur_user->log_entries, cur_prog->used);fflush(stdout);
#endif
			if(cur_prog->already_logged!='Y') {
				LOG.writeline(cur_user->username, cur_prog->name, 'K');
				cur_prog->already_logged='Y';
			} else {
				LOG.writeline(cur_user->username, cur_prog->name, 'N');
			}
			cur_prog->used+=(cur_user->log_entries)/10;
			A3+=0.1;
		}
	}
	// day of the week
	A4=cur_user->log_entries/cur_user->known_daysofweek[tm_->tm_wday];  // int
   
	// hour of the day
	A5=cur_user->log_entries/cur_user->known_timeintervals[tm_->tm_hour]; // int
	
	// ///////////////////////////////
	// LAYER 1
	// ///////////////////////////////
	
	// h1
	if(cur_user->avg_house_chng_factor==0 && cur_user->avg_etage_chng_factor==0 && cur_user->avg_room_chng_factor==0)
		h1=cur_user->avg_x_chng_factor+cur_user->avg_y_chng_factor;	// 0 || 1 || 2
	else
		h1=(cur_user->avg_x_chng_factor+cur_user->avg_y_chng_factor)/2;	// 0 || 1
	// h2
	if(cur_user->avg_house_chng_factor) {
		h2=cur_user->avg_house_chng_factor;				// 0 || 2
	} else {
		// building: not changed
		if(cur_user->avg_etage_chng_factor) {
			h2=cur_user->avg_etage_chng_factor;			// 0 || 1
		} else {
			// bilding & etage: not changed
			h2=cur_user->avg_room_chng_factor;			// 0 || 1
		}
	}
   // h3 (hour of the day & day of the week)
   // 1. time of the day
	if(A5>10.0f) {
		h3=2;
		atk_time=1;
	} else if(A5>5.0f) {
		h3=1;
		atk_time=1;
	}
   // 2. day of the week
   if(A4>10.0f) {
      h3+=2;
      atk_date=1;
   } else if(A4>5.0f) {
      h3+=1;
      atk_date=1;
   }
   
	// h4 (used programs)
	if(A3>0.1) {
		h4=2;
		atk_tools=1;
	} else if(A3==0.1) {
		h4=1;
		atk_tools=1;
	}
		
	// ///////////////////////////////
	// POs (pre outputs)
	// ///////////////////////////////
	if((h1+h2)>1)
		po1=2;
	else if((h1+h2)==1)
		po1=1;
	
	if((h3+h4)>1)
		po2=2;
	else if((h3+h4)==1)
		po2=1;
		
	// ///////////////////////////////
	// OUTPUT
	// ///////////////////////////////
	out=po1+po2;
	cur_user->cur_atklvl=out;

#ifdef DEBUG
	printf("CAL_OF(%10s):\n\t\th1=%f\t\th3=%f\n\t\th2=%f\t\th4=%f\t\t(prog=%f)\n\t\tpo1=%f\t\tpo2=%f\t\tOUT:=%f\n",
		cur_user->username, h1,h3, h2,h4,A3, po1,po2, out);
#endif
	// THIS IS IMPORTANT: delete the 'already_changed' flag!
	cur_user->already_changed=false;
}

// 
// i know cur_user is set to the right one because i called lookup_user
// before. this function will do a lookup on the tool and if the tool
// is new, it will create a new entry for it.
// 

void
Users::lookup_tool(char *progname)
{
	char *local_nameptr=0;
	
	if(cur_user->users_progs==NULL) {
		cur_user->users_progs=(struct proglist *)malloc(sizeof(struct proglist));
		if(!cur_user->users_progs) {
			syslog(SYSL_ERR, "mem low. unable to create new prog-entry. exiting.");
			exit(1);
		}
		bzero(cur_user->users_progs, sizeof(struct proglist));
		cur_prog=cur_user->users_progs;
	
		// prepare the cmdname
		local_nameptr=(char *)malloc(strlen(progname)+1);
		if(!local_nameptr) {
			syslog(SYSL_ERR, "mem low. unable to create new prog-entry. exiting.");
			exit(1);
		}
		bzero(local_nameptr, strlen(progname)+1);
		strncpy(local_nameptr, progname, strlen(progname));
		
		// set values
		cur_prog->name=local_nameptr;
		cur_prog->used=1;
		//cur_prog->usage_factor=(float)cur_user->log_entries/(float)cur_prog->used;
		
		// inc the number of progs
		cur_user->num_of_progs++;
		
	} else {
		// first: try to find an entry for this tool
		for(cur_prog=cur_user->users_progs; cur_prog; cur_prog=cur_prog->next) {
			if(strcmp(cur_prog->name, progname)==0) {
				/* found it */
				cur_prog->used++;
				//cur_prog->usage_factor=(float)cur_prog->used/(float)cur_user->log_entries;
				return;
			}
		}
		
		// we didn't found it. lets create an new entry
		for(cur_prog=cur_user->users_progs; cur_prog->next; cur_prog=cur_prog->next)
			;
		
		cur_prog->next=(struct proglist *)malloc(sizeof(struct proglist));
		if(!cur_prog->next) {
			syslog(SYSL_ERR, "mem low. unable to create new prog-entry. exiting.");
			exit(1);
		}
		cur_prog=cur_prog->next;
		bzero(cur_prog, sizeof(struct proglist));
	
		// prepare the cmdname
		local_nameptr=(char *)malloc(strlen(progname)+1);
		if(!local_nameptr) {
			syslog(SYSL_ERR, "mem low. unable to create new prog-entry. exiting.");
			exit(1);
		}
		bzero(local_nameptr, strlen(progname)+1);
		strncpy(local_nameptr, progname, strlen(progname));
		
		// set values
		cur_prog->name=local_nameptr;
		cur_prog->used=1;
		//cur_prog->usage_factor=(float)cur_user->log_entries/(float)cur_prog->used;
		
		// inc the number of progs
		cur_user->num_of_progs++;
	}
}

//
// set cur_user to the entry of the user. if the user is new, create
// a new entry and set cur_user too.
//

#define BZERO_USERS_DATA		\
	bzero(cur_user, sizeof(user));

void
Users::lookup_user(char *username)
{
	const time_t x=time(NULL);

	tm_=localtime(&x);
		
	for(cur_user=user_db; cur_user!=NULL; cur_user=cur_user->next)
		if(strcmp(cur_user->username, username)==0) {
			if(cur_user->already_changed==false) {
				cur_user->known_timeintervals[tm_->tm_hour]++;
            cur_user->known_daysofweek[tm_->tm_wday]++;
				// log_entries
				cur_user->log_entries++;
			}
			cur_user->already_changed=true;
			return;
		}

	if(user_db==NULL) {
		user_db=(user *)malloc(sizeof(user));
		if(user_db==NULL) {
			syslog(SYSL_ERR, "mem low. unable to create new user-entry. exiting.");
			exit(1);
		}
		bzero(user_db, sizeof(user));
		cur_user=user_db;
		BZERO_USERS_DATA
		strncpy(cur_user->username, username, 512);
		cur_user->log_entries=1;
		cur_user->known_timeintervals[tm_->tm_hour]++;
      cur_user->known_daysofweek[tm_->tm_wday]++;
		cur_user->already_changed=true;
	} else {
		for(cur_user=user_db; cur_user->next!=NULL; cur_user=cur_user->next)
			;
		cur_user->next=(user *)malloc(sizeof(user));
		if(cur_user->next==NULL) {
			syslog(SYSL_ERR, "mem low. unable to create new user-entry. exiting.");
			exit(1);
		}
		bzero(cur_user->next, sizeof(user));
		cur_user=cur_user->next;
		BZERO_USERS_DATA
		strncpy(cur_user->username, username, 512);
		cur_user->log_entries=1;
		cur_user->known_timeintervals[tm_->tm_hour]++;
      cur_user->known_daysofweek[tm_->tm_wday]++;
		cur_user->already_changed=true;
	}
}

void
Users::load_user_config()
{
	FILE *fp;
	usrdb_ffmt header;
	usrdb_prg_ffmt progheader;
	char *house;
	char *progname;
	unsigned int prog_counter;
	
	if((fp=fopen(USERDBFILE, "r"))==NULL) {
		fprintf(stderr, "unable to open userfile " USERDBFILE ", seems to be the first start of fupids2.\n");
		return;
	}
	
	printf("[reading user-database]\n");
	fflush_out();
	
	while(fread(&header, sizeof(header), 1, fp)) {
		if(!(house=(char *)malloc(header.last_house_len+1))) {
			perror("malloc");
			exit(1);
		}
		bzero(house, header.last_house_len+1);
		fread(house, header.last_house_len, 1, fp);
#ifdef DEBUG
		printf("-------------------username=%s, house=%s\n", header.username, house);fflush_out();
#endif
		// add the user in the user_db
		lookup_user(header.username);
		cur_user->last_x=header.last_x;
		cur_user->last_y=header.last_y;
		cur_user->last_room=header.last_room;
		cur_user->last_etage=header.last_etage;
		cur_user->already_changed=false;
		memcpy(cur_user->known_timeintervals, header.known_timeintervals, sizeof(header.known_timeintervals));
      memcpy(cur_user->known_daysofweek, header.known_daysofweek, sizeof(header.known_daysofweek));

		// do NOT set num_of_progs, lookup_tool will increment this for every tool on its own
		//cur_user->num_of_progs=header.num_of_progs;
		cur_user->log_entries=header.log_entries;
		cur_user->last_house=house;		
		
		for(prog_counter=header.num_of_progs; prog_counter; prog_counter--) {
			fread(&progheader, sizeof(progheader), 1, fp);
			if(!(progname=(char *)malloc(progheader.namelen+1))) {
				perror("malloc");
				exit(1);
			}
			bzero(progname, progheader.namelen+1);
			fread(progname, progheader.namelen, 1, fp);
#ifdef DEBUG
			printf("\t\t------tool=%s\n", progname);
#endif
			// add the entry in users proglist
			lookup_tool(progname);
			cur_prog->used=progheader.used;
			cur_prog->already_logged=progheader.already_logged;
		}
	}

	fclose(fp);
}

/*typedef struct {
	unsigned short last_x;
	unsigned short last_y;
	unsigned int last_room;
	unsigned int last_etage;
	unsigned int num_of_progs;
	unsigned long long log_entries;
	char username[512+1];
	unsigned int last_house_len;
} usrdb_ffmt;*/

/*typedef struct {
	unsigned int namelen;
	unsigned long long used;
} usrdb_prg_ffmt;*/

void
Users::save_user_config()
{
	FILE *fp;
	usrdb_ffmt header;
	usrdb_prg_ffmt progheader;
	proglist *prog_ptr;

	/* do nothing if there is no user in our DB */
	if(!user_db)
		return;
	
	/* if the current config exists, delete it */
	link(USERDBFILE, USERDBFILE ".old");
	unlink(USERDBFILE);
	
	if(!(fp=fopen(USERDBFILE, "w+"))) {
		syslog(SYSL_ERR, "unable to safe user-database. error while fopen(" USERDBFILE ")");
		exit(1);
	}
	
	for(cur_user=user_db; cur_user; cur_user=cur_user->next) {
		// user hdr
		header.last_x=cur_user->last_x;
		header.last_y=cur_user->last_y;
		header.last_room=cur_user->last_room;
		header.last_etage=cur_user->last_etage;
		header.num_of_progs=cur_user->num_of_progs;
		header.log_entries=cur_user->log_entries;
		header.last_house_len=strlen(cur_user->last_house);
		memcpy(header.known_timeintervals, cur_user->known_timeintervals, sizeof(cur_user->known_timeintervals));
      memcpy(header.known_daysofweek, cur_user->known_daysofweek, sizeof(cur_user->known_daysofweek));
		strncpy(header.username, cur_user->username, 512);
		fwrite(&header, sizeof(usrdb_ffmt), 1, fp);
		
		// last_house-string
		fwrite(cur_user->last_house, strlen(cur_user->last_house), 1, fp);
		
		// tools
		for(prog_ptr=cur_user->users_progs; prog_ptr; prog_ptr=prog_ptr->next) {
			// tool hdr
			progheader.used=prog_ptr->used;
			progheader.namelen=strlen(prog_ptr->name);
			progheader.already_logged=prog_ptr->already_logged;
			fwrite(&progheader, sizeof(usrdb_prg_ffmt), 1, fp);
			
			// tool's name-string
			fwrite(prog_ptr->name, strlen(prog_ptr->name), 1, fp);
		}
	}
	fflush(fp);
	fclose(fp);
#ifdef DEBUG
	printf("[user database successfully saved]\n");
#endif
	fflush(stdout);
}

