/***************************************************************************
 *   Copyright (C) 2005-2006 by Steffen Wendzel                            *
 *   cdp {at} doomed-reality {dot} org                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "fupids2.h"

Workstations::Workstations()
{
	ws_db=cur_ws=NULL;
	load_base_config();
}

void
Workstations::dump_wslist()
{
	for(cur_ws=ws_db; cur_ws!=NULL; cur_ws=cur_ws->next)
		printf("FQDN=%s\n", cur_ws->fqdn);
}

// 
// set cur_ws to the found WS if it exists and return 1 on success
// 

int
Workstations::search_ws(char *fqdn)
{
	for(cur_ws=ws_db; cur_ws!=NULL; cur_ws=cur_ws->next)
		if(strcmp(cur_ws->fqdn, fqdn)==0)
			return 1;

	return -1;
}

void
Workstations::load_base_config()
{
	FILE *fp;
	int len;
	char *buf;
	int i,k;
	char *hostname;
	char *house;
	int etage;
	int room;
	char x,y;
	ws_info *ws;

#define CHK_CUR_BUF_NULL if(buf[i]=='\0') { fprintf(stderr, "syntax error: unexpected end of file fupids2.conf.\n"); exit(1); }

	printf("[loading base-config]\n"), fflush_out();
		
	if((fp=fopen(CONFIGFILE, "r"))==NULL) {
		perror("Failed to open config file " CONFIGFILE);
		exit(1);
	}
	len=get_openfilelen(fp);
	if((buf=(char *)malloc((sizeof(char)*len)+1))==NULL) {
		perror("malloc");
		exit(1);
	}
	bzero(buf, len+1);
	if(fread(buf, len, 1, fp)!=1) {
		perror("fread");
		exit(1);
	}
	fclose(fp);
	
	/* Format: HOST yorick.sun  IN T,2,209 w,f */
	for(i=0; i<len; i++) {
		if(buf[i]=='#') {	/* comment */
			while(buf[i]!='\n' && i<len)
				i++;
		} else if(buf[i]==' ' || buf[i]=='\r' || buf[i]=='\t' || buf[i]=='\n') {
		} else if((i+6)<len && buf[i]=='H' && buf[i+1]=='O' && buf[i+2]=='S' && buf[i+3]=='T') {
			i+=4;
			if(buf[i]!=' ') {
				fprintf(stderr, "syntax error in fupids2.conf near HOST-Keyword.\n");
				exit(1);
			}
			i++;
			k=i;
			while(buf[k]!=' ' && buf[k]!='\0') {
				k++;
			}
			if(k==i) {
				fprintf(stderr, "syntax error: no hostname given after HOST-Keyword.\n");
				exit(1);
			} else {
				hostname=(char *)malloc(k-i+1);
				if(hostname==NULL) {
					perror("malloc");
					exit(1);
				}
				bzero(hostname, k-i+1);
				strncpy(hostname, (buf+i), k-i);
			}
			i=k;
			
			while(buf[i]==' ' || buf[i]=='\t')
				i++;
			CHK_CUR_BUF_NULL
			
			if((i+4)>=len || buf[i]!='I' || buf[i+1]!='N' || buf[i+2]!=' ') {
				fprintf(stderr, "syntax error in fupids2.conf: no IN-keyword found.\n");
				exit(1);
			}
			i+=3;
			CHK_CUR_BUF_NULL
			while(buf[i]==' ' || buf[i]=='\t')
				i++;
			CHK_CUR_BUF_NULL
			
			/* T,2,209 w,f */
			k=i;
			while(buf[k]!=',' && buf[k]!='\0') {
				k++;
			}
			if(k==i) {
				fprintf(stderr, "syntax error: no hostname given after IN-Keyword.\n");
				exit(1);
			} else {
				house=(char *)malloc(k-i+1);
				if(house==NULL) {
					perror("malloc");
					exit(1);
				}
				bzero(house, k-i+1);
				strncpy(house, (buf+i), k-i);
			}
			i=k+1;
			
			etage=atoi(buf+i);
			while(buf[i]!='\0' && buf[i]!=',')
				i++;
			i++;
			CHK_CUR_BUF_NULL
			
			room=atoi(buf+i);
			while(buf[i]!='\0' && ((buf[i]>='1' && buf[i]<='9') || buf[i]=='0' || buf[i]==' ' || buf[i]=='\t'))
				i++;
			CHK_CUR_BUF_NULL
			
			switch(buf[i]) {
			case 'w':
			case 'm':
			case 'f':
				x=buf[i];
				break;
			default:
				fprintf(stderr, "unexpected EOf in fupids2.conf. [441]\n");
				exit(1);
			}
			if(buf[i+1]=='\0' || buf[i+2]=='\0') {
				fprintf(stderr, "unexpected EOF in fupids2.conf. [442]\n");
				exit(1);
			}
			i+=2;
			switch(buf[i]) {
			case 'b':
			case 'm':
			case 'f':
				y=buf[i];
				break;
			default:
				fprintf(stderr, "unexpected EOf in fupids2.conf. [443]\n");
				exit(1);
			}
			
			printf("\t\t------initializing host %s, building %s, et %i, room %i, %c,%c... ", hostname, house, etage, room, x, y);
			ws=(ws_info *)malloc(sizeof(ws_info));
			if(ws==NULL) {
				perror("malloc");
				exit(1);
			}
			ws->fqdn=hostname;
			ws->house=house;
			ws->etage=etage;
			ws->room=room;
			ws->x=x;
			ws->y=y;
			ws->next=NULL;

			if(ws_db==NULL) {
				ws_db=ws;
			} else {
				ws_info *ws_ptr;
				for(ws_ptr=ws_db; ws_ptr->next!=NULL; ws_ptr=ws_ptr->next)
					;
				ws_ptr->next=ws;
			}
			printf("[ok]\n");
			fflush(stdout);
			
			/* set all back to unused values (not needed) */
			hostname=NULL;
			house=NULL;
			etage=room=0;
			x=y='_';
		} else {
			fprintf(stderr, "syntax error in fupids2.conf [444] buf='%s'\n", (buf+i));
			exit(1);
		}
	}
}

